@@ -103,7 +103,7 @@ class Runner
       @basepath + 'src/port/win32env.c', # Win32 only
       @basepath + 'src/port/win32security.c', # Win32 only
       @basepath + 'src/port/gettimeofday.c', # Win32 only
-      @basepath + 'src/port/strlcpy.c', # Not needed and conflicts with available function
+      @basepath + 'src/port/strnlen.c', # Not needed and conflicts with available function
       @basepath + 'src/port/strlcat.c', # Not needed and conflicts with available function
       @basepath + 'src/port/unsetenv.c', # Not needed and conflicts with available function
       @basepath + 'src/port/getaddrinfo.c', # Not needed and conflicts with available function
@@ -214,14 +214,33 @@ class Runner
 
   def analyze_file(file)
     index = FFI::Clang::Index.new(true, true)
-    translation_unit = index.parse_translation_unit(file, [
+    flags = [
       '-I', @basepath + 'src/include',
       '-I', '/usr/local/opt/openssl/include',
       '-I', `xcrun --sdk macosx --show-sdk-path`.strip + '/usr/include',
       '-DDLSUFFIX=".bundle"',
       '-g',
-      '-DUSE_ASSERT_CHECKING'
-    ])
+      '-ferror-limit=0',
+      '-DUSE_ASSERT_CHECKING',
+      # EXEC_BACKEND is used on Windows, and can always be safely set during code analysis
+      '-DEXEC_BACKEND',
+    ]
+
+    # For certain files, use WIN32 define - we can't always do this since it pulls unnecessary code in other cases
+    if file == @basepath + 'src/backend/utils/error/elog.c' || file == @basepath + 'src/backend/utils/mb/mbutils.c'
+      flags << '-DWIN32'
+      flags << '-D__CYGWIN__' # Avoid pulling in win32_port.h (which includes a bunch of system headers we don't actually have)
+    end
+
+    # To override built-ins, we must avoid pulling in any system headers, so pretend we already defined c.h
+    if file == @basepath + 'src/port/strlcpy.c'
+      flags << '-DC_H'
+      flags << '-DHAVE_DECL_STRLCPY=0'
+      flags << '-Dsize_t=unsigned'
+    end
+
+    translation_unit = index.parse_translation_unit(file, flags)
+
     cursor = translation_unit.cursor
 
     func_cursor = nil
@@ -363,7 +382,7 @@ class Runner
   end
 
   def special_include_file?(filename)
-    filename[/\/(reg(c|e)_[\w_]+|guc-file|qsort_tuple|repl_scanner|levenshtein|bootscanner|like_match)\.c$/] || filename[/\/[\w_]+_impl.h$/]
+    filename[/\/(reg(c|e)_[\w_]+|guc-file|qsort_tuple|repl_scanner|levenshtein|bootscanner|like_match)\.c$/] || filename[/\/[\w_]+\.funcs.c$/] || filename[/\/[\w_]+_impl.h$/]
   end
 
   def write_out
@@ -427,20 +446,22 @@ class Runner
       end
       all_thread_local_variables += file_thread_local_variables
 
-      if special_include_file?(filename)
-        out_name = File.basename(filename)
-      else
+      unless special_include_file?(filename)
         out_name = filename.gsub(%r{^#{@basepath}}, '').gsub('/', '_')
+        File.write(@out_path + out_name, str)
       end
-
-      File.write(@out_path + out_name, str)
     end
 
     #return
 
-    @include_files_to_output.each do |include_file|
-      next if special_include_file?(include_file)
+    additional_includes = Dir.glob(@basepath + 'src/include/storage/dsm_impl.h') +
+      Dir.glob(@basepath + 'src/include/port/atomics/**/*.h') +
+      Dir.glob(@basepath + 'src/include/port/win32/**/*.h') +
+      Dir.glob(@basepath + 'src/include/port/win32_msvc/**/*.h') +
+      Dir.glob(@basepath + 'src/include/port/win32.h') +
+      Dir.glob(@basepath + 'src/include/port/win32_port.h')
 
+    (@include_files_to_output + additional_includes).each do |include_file|
       if include_file.start_with?(@basepath + 'src/include')
         out_file = @out_path + include_file.gsub(%r{^#{@basepath}src/}, '')
       else
@@ -449,7 +470,7 @@ class Runner
 
       code = File.read(include_file)
       all_thread_local_variables.each do |variable|
-        code.gsub!(/(PGDLLIMPORT|extern)\s+(const|volatile)?\s*(\w+)\s+(\*{0,2})#{variable}(\[\])?;/, "\\1 __thread \\2 \\3 \\4#{variable}\\5;")
+        code.gsub!(/(extern\s+)(PGDLLIMPORT\s+)?(const\s+)?(volatile\s+)?(\w+)\s+(\*{0,2})#{variable}(\[\])?;/, "\\1\\2__thread \\3\\4\\5 \\6#{variable}\\7;")
       end
 
       FileUtils.mkdir_p File.dirname(out_file)
@@ -539,6 +560,20 @@ make_return_stmt(int location)
 }
 )) # We're always working with fn_rettype = VOIDOID, due to our use of plpgsql_compile_inline
 
+# Mocks REQUIRED for Windows support
+runner.mock('write_stderr', %(
+void
+write_stderr(const char *fmt,...)
+{
+	va_list	ap;
+	va_start(ap, fmt);
+	vfprintf(stderr, fmt, ap);
+	fflush(stderr);
+	va_end(ap);
+}
+)) # Avoid pulling in write_console/write_eventlog, and instead always output to stderr (like on POSIX)
+runner.mock('should_output_to_client', 'static inline bool should_output_to_client(int elevel) { return false; }') # Avoid pulling in postmaster.c, which has a bunch of Windows-specific code hidden behind a define
+
 ## ---
 
 # SQL Parsing
@@ -586,6 +621,23 @@ runner.deep_resolve('MemoryContextAllocExtended')
 # Other required functions
 runner.deep_resolve('pg_printf')
 
+# Retain these functions for optional 32-bit support
+# (see BITS_PER_BITMAPWORD checks in bitmapset.c)
+runner.deep_resolve('pg_leftmost_one_pos32')
+runner.deep_resolve('pg_rightmost_one_pos32')
+runner.deep_resolve('pg_popcount32')
+
+# Required for Windows support
+runner.deep_resolve('newNodeMacroHolder')
+runner.deep_resolve('pg_leftmost_one_pos')
+runner.deep_resolve('pg_rightmost_one_pos')
+runner.deep_resolve('pg_number_of_ones')
+runner.deep_resolve('GetMessageEncoding')
+runner.deep_resolve('strlcpy')
+runner.deep_resolve('pg_signal_queue')
+runner.deep_resolve('pg_signal_mask')
+runner.deep_resolve('pgwin32_dispatch_queued_signals')
+
 runner.write_out
 
 #puts runner.unresolved.inspect
