@@ -17,6 +17,8 @@
  * - pg_mbstrlen_with_len
  * - pg_mblen
  * - SetDatabaseEncoding
+ * - GetMessageEncoding
+ * - MessageEncoding
  *--------------------------------------------------------------------
  */
 
@@ -105,6 +107,7 @@ static __thread const pg_enc2name *ClientEncoding = &pg_enc2name_tbl[PG_SQL_ASCI
 
 static __thread const pg_enc2name *DatabaseEncoding = &pg_enc2name_tbl[PG_SQL_ASCII];
 
+static __thread const pg_enc2name *MessageEncoding = &pg_enc2name_tbl[PG_SQL_ASCII];
 
 
 /*
@@ -595,7 +598,11 @@ GetDatabaseEncodingName(void)
  * not attached to a database, and under a database encoding lacking iconv
  * support (MULE_INTERNAL).
  */
-
+int
+GetMessageEncoding(void)
+{
+	return MessageEncoding->encoding;
+}
 
 
 /*
@@ -770,68 +777,6 @@ report_invalid_encoding(int encoding, const char *mbstr, int len)
  * null. Returns NULL iff failed. Before MessageEncoding initialization, "str"
  * should be ASCII-only; this will function as though MessageEncoding is UTF8.
  */
-WCHAR *
-pgwin32_message_to_UTF16(const char *str, int len, int *utf16len)
-{
-	int			msgenc = GetMessageEncoding();
-	WCHAR	   *utf16;
-	int			dstlen;
-	UINT		codepage;
 
-	if (msgenc == PG_SQL_ASCII)
-		/* No conversion is possible, and SQL_ASCII is never utf16. */
-		return NULL;
-
-	codepage = pg_enc2name_tbl[msgenc].codepage;
-
-	/*
-	 * Use MultiByteToWideChar directly if there is a corresponding codepage,
-	 * or double conversion through UTF8 if not.  Double conversion is needed,
-	 * for example, in an ENCODING=LATIN8, LC_CTYPE=C database.
-	 */
-	if (codepage != 0)
-	{
-		utf16 = (WCHAR *) palloc(sizeof(WCHAR) * (len + 1));
-		dstlen = MultiByteToWideChar(codepage, 0, str, len, utf16, len);
-		utf16[dstlen] = (WCHAR) 0;
-	}
-	else
-	{
-		char	   *utf8;
-
-		/*
-		 * XXX pg_do_encoding_conversion() requires a transaction.  In the
-		 * absence of one, hope for the input to be valid UTF8.
-		 */
-		if (IsTransactionState())
-		{
-			utf8 = (char *) pg_do_encoding_conversion((unsigned char *) str,
-													  len,
-													  msgenc,
-													  PG_UTF8);
-			if (utf8 != str)
-				len = strlen(utf8);
-		}
-		else
-			utf8 = (char *) str;
-
-		utf16 = (WCHAR *) palloc(sizeof(WCHAR) * (len + 1));
-		dstlen = MultiByteToWideChar(CP_UTF8, 0, utf8, len, utf16, len);
-		utf16[dstlen] = (WCHAR) 0;
-
-		if (utf8 != str)
-			pfree(utf8);
-	}
-
-	if (dstlen == 0 && len > 0)
-	{
-		pfree(utf16);
-		return NULL;			/* error */
-	}
-
-	if (utf16len)
-		*utf16len = dstlen;
-	return utf16;
-}
 
 #endif							/* WIN32 */
