@@ -398,6 +398,16 @@ const char* tests[] = {
   "CREATE PROCEDURE returns_one() LANGUAGE sql BEGIN ATOMIC RETURN 1; END",
   "CREATE PROCEDURE updates_and_returns_one() LANGUAGE sql BEGIN ATOMIC UPDATE tbl SET a = 1; RETURN 1; END",
   "SELECT 1 FROM tbl LIMIT COALESCE($1, $2)",
+  "SELECT (false AND true) IS FALSE",
+  "SELECT a = (true IS FALSE)",
+  "ALTER TABLE a ENABLE TRIGGER b",
+  "ALTER TABLE a ENABLE TRIGGER ALL",
+  "ALTER TABLE a ENABLE TRIGGER USER",
+  "ALTER TABLE a DISABLE TRIGGER b",
+  "ALTER TABLE a DISABLE TRIGGER ALL",
+  "ALTER TABLE a DISABLE TRIGGER USER",
+  "CREATE INDEX myindex ON public.mytable USING btree (col1, (col2::varchar) varchar_pattern_ops)",
+  "SELECT * FROM CAST(1 AS text)"
 };
 
 size_t testsLength = __LINE__ - 4;
